/*
 * MPU6050.c
 *
 * Created: 05.11.2014 14:53:44
 *  Author: Marcel
 */

#define F_CPU 12000000UL

#include <util/delay.h>
#include "i2c_master.h"

#define MPU6050_ADDRESS_WRITE	0xD0
#define MPU6050_ADDRESS_READ	0xD1

int ax = 0;
int ay = 0;
int az = 0;
int gx = 0;
int gy = 0;
int gz = 0;

int32_t gx_Offset_Sum = 0;
int32_t gy_Offset_Sum = 0;
int32_t gz_Offset_Sum = 0;

int gx_Offset = 0;
int gy_Offset = 0;
int gz_Offset = 0;

void init_MPU6050(){
	
	i2c_start(MPU6050_ADDRESS_WRITE);	// Set Clock Source
	i2c_write(0x6B);
	i2c_write(0x80);
	i2c_stop();
	
	_delay_ms(50);
	
	i2c_start(MPU6050_ADDRESS_WRITE);	// Set Clock Source and disable Sleep Modus
	i2c_write(0x6B);
	i2c_write(0x03);
	i2c_stop();
	
	i2c_start(MPU6050_ADDRESS_WRITE);	// Disable Input Pin for Data Sync
	i2c_write(0x1A);
	i2c_write(0x00);
	i2c_stop();
	
	i2c_start(MPU6050_ADDRESS_WRITE);	// Set Gyro Scale Range: Full scale set to 2000 deg/sec
	i2c_write(0x1B);
	i2c_write(0x18);
	//i2c_write(0x03);
	i2c_stop();
	
	i2c_start(MPU6050_ADDRESS_WRITE);	// Set Accel Range
	i2c_write(0x1C);
	i2c_write(0x10);
	i2c_stop();
	
	/*i2c_start(MPU6050_ADDRESS_WRITE);	// Disable Sleep Modus
	i2c_write(0x6B);
	i2c_write(0x0);
	i2c_stop();*/
}

void ReadDataAccel(){
	
	i2c_start(MPU6050_ADDRESS_WRITE);
	i2c_write(0x3B);
	i2c_rep_start(MPU6050_ADDRESS_READ);
	
	int axHigh = i2c_readAck();
	int axLow = i2c_readAck();
	
	int ayHigh = i2c_readAck();
	int ayLow = i2c_readAck();
	
	int azHigh = i2c_readAck();
	int azLow = i2c_readNak();
	
	i2c_stop();
	
	ax = axHigh << 8 | axLow;
	ay = ayHigh << 8 | ayLow;
	az = azHigh << 8 | azLow;
}

void ReadDataGyro(){
	
	i2c_start(MPU6050_ADDRESS_WRITE);
	i2c_write(0x43);
	i2c_rep_start(MPU6050_ADDRESS_READ);
	
	int gxHigh = i2c_readAck();
	int gxLow = i2c_readAck();
	
	int gyHigh = i2c_readAck();
	int gyLow = i2c_readAck();
	
	int gzHigh = i2c_readAck();
	int gzLow = i2c_readNak();
	
	i2c_stop();
	
	gx = (gxHigh << 8 | gxLow) * 1.0;// - gx_Offset;
	gy = (gyHigh << 8 | gyLow) * 1.0;// - gy_Offset;
	gz = (gzHigh << 8 | gzLow) * 1.0;// - gz_Offset;
}

void Calibrate_Gyro()
{
	for(int x = 0; x < 1000; x++)
	{
		i2c_start(MPU6050_ADDRESS_WRITE);
		i2c_write(0x43);
		i2c_rep_start(MPU6050_ADDRESS_READ);
	
		int gxHigh = i2c_readAck();
		int gxLow = i2c_readAck();
	
		int gyHigh = i2c_readAck();
		int gyLow = i2c_readAck();
	
		int gzHigh = i2c_readAck();
		int gzLow = i2c_readNak();
	
		i2c_stop();
	
		gx_Offset_Sum += gxHigh << 8 | gxLow;
		gy_Offset_Sum += gyHigh << 8 | gyLow;
		gz_Offset_Sum += gzHigh << 8 | gzLow;
 
		_delay_ms(1);
	}
	
	gx_Offset = gx_Offset_Sum / 1000;
	gy_Offset = gy_Offset_Sum / 1000;
	gz_Offset = gz_Offset_Sum / 1000;
}

int GetGyroX(){
	
	return gx;
}

int GetGyroY(){
	
	return gy;
}

int GetGyroZ(){
	
	return gz;
}

int GetAccelX(){
	
	return ax;
}

int GetAccelY(){
	
	return ay;
}

int GetAccelZ(){
	
	return az;	
}
